;***********************************************************************
;* File Name:           avr_idle.asm
;* Title:               FPSLIC Sample Design 1 (Assembly)
;* Version:             1.1
;* Last Updated:        May 24, 2004
;* Target Device:       AT94K05/10/40
;* Purpose:	            To put the FPSLIC AVR Core to Sleep in Idle Mode
;*				and wake it upon a Timer/Counter1 Overflow 
;*				Interrupt 
;* Hardware used:		ATSTK94 FPSLIC Starter Kit. The Starter Board 
;*				can be used to observe the behavior of the 
;*				FPSLIC as its LED's go from flashing 55/AAh in 
;*				the active state to OFF in the sleep state 
;***********************************************************************

;***** Include Files  *****
.include "AT94KDEF.INC"

;***** Global Registers *****
;* Define Symbolic Names for Registers
;* Remember immediate operations (i.e. ldi r16, 0x05) available on r16 
;* through r31

.def    	temp      = r17   	;Temporary Storage Register
.def    	counter1  = r20   	;Counter used for delay
.def    	counter2  = r21   	;Counter used for delay
.def    	counter3  = r22   	;Counter used for delay
.def    	counter4  = r23   	;Register to set 55/AAh toggle count
.def   	pattern   = r24   	;Register used to load the pattern

;***** Interrupt Vectors *****
.org RESETaddr
   rjmp RESET                 	;Reset Interrupt

.org TIM1_OVFaddr
   rjmp TIM1_OVF              	;Timer1 Overflow Interrupt

; ----------------------------------------------------------------------
; Interrupt Service Routine
; ----------------------------------------------------------------------
TIM1_OVF:                     	;Timer1 Overflow Interrupt Service Routine
     
Delay_ISR: 				
   inc      counter1			;After waking from sleep mode as a result 
   cpi 	counter1, 0xAA		;of Timer/Counter 1 interrupt, this loop 	
   brne 	delay_ISR			;is used to delay the indication of FPSLIC 
   clr 	counter1			;being in active mode by sending 00h to 
   inc 	counter2			;LEDs available on ATSTK94 board. This 
   cpi 	counter2, 0xAA		;loop is strictly used for display 
   brne 	delay_ISR			;purpose, in case of measuring idle mode 
   clr 	counter2			;power consumption comment this code.


   	reti    
		                
; ----------------------------------------------------------------------
; Initialization Routine
; ----------------------------------------------------------------------
RESET:                        	;Initialize Stack Pointer and DDRD
   	ldi 	temp,0x0f               
   	out 	SPH,temp          	;Initialize Stack Pointer-> 0fff 
   	ldi 	temp,0xff               
   	out 	SPL,temp           
   	out 	DDRD,temp         	;Configure PortD pins as output pins
   	ldi 	pattern, 0x55           

   	ldi 	temp, (1<<TOIE1)  	;Enable Timer/Counter1 Overflow interrupt
   	out 	TIMSK,temp             
   	clr 	Temp                   
   	out 	TCNT1H,temp       	;Count value is at 0000
   	out 	TCNT1L,temp            
   	ldi 	Temp,(1<<CS12)+(0<<CS11)+(1<<CS10)
   	out 	TCCR1B,temp       	;Timer Pre-scalar set to ck/1024

   	ldi 	Temp, (1<<SE)+(0<<SM1)+(0<<SM0)
   	out 	MCUR, temp        	;Enable Sleep in Idle Mode
   
   sei                        	;Enable global interrupts

; ----------------------------------------------------------------------
; Main Routine
; ----------------------------------------------------------------------
MAIN:
   	clr 	counter1          	;Initialize Counters
   	clr 	counter2
   	clr 	counter3
   	clr 	counter4
   	out 	PORTD,pattern     	;Display 55h on LEDs
        
delay:					;Embedded delay loop to slow the display 
   	inc 	counter3	      	;of AAh & 55h pattern, making it 		
   	cpi 	counter3, 0xff		;observable on STK94 starter board
   	brne 	delay
   	clr 	counter3
   	inc 	counter2			
   	cpi 	counter2, 0xff
   	brne 	delay
   	clr 	counter2
   	inc 	counter1
   	cpi 	counter1, 0x0f
   	brne 	delay
   	clr 	counter1
   	com 	pattern           	;One's complements the pattern(55h-> AAh)
   	out 	PORTD,pattern             
   	inc 	counter4
   	cpi 	counter4, 0x0A		;55/AAh toggles 10 times
   	brne 	delay
   	clr 	counter4       
    
   	clr 	temp  			;LEDs are switched off to infer idle sleep 
   	out 	PORTD,temp 			;mode on STK94 starter board    
   	sleep    				;Go to sleep (wait for Timer/Counter1 int)       
   	rjmp 	MAIN				;Jump to MAIN to repeat displaying 55/AAh
						;pattern after waking-up from sleep

