;***********************************************************************
;* File Name:           avr_powerdown.asm
;* Title:               FPSLIC Sample Design 1 (Assembly)
;* Version:             1.1
;* Last Updated:        May 26, 2004
;* Target Device:       AT94K05/10/40
;* Purpose:	            To put the FPSLIC AVR Core to Sleep in Power-
;*				Down Mode and wake it upon an External Interrupt
;* Hardware used:		ATSTK94 FPSLIC Starter Kit. The Starter Board 
;*				can be used to observe the behavior of the 
;*				FPSLIC as its LED's go from flashing 55/AAh in 
;*				the active state to OFF in the sleep state 
;***********************************************************************

;***** Include Files  *****
.include "AT94KDEF.INC"

;***** Global Registers *****
;* Define Symbolic Names for Registers
;* Remember immediate operations (i.e. ldi r16, 0x05) available on r16 
;* through r31

.def    	temp      = r16   	;Temporary Storage Register
.def    	counter1  = r21   	;Registers used for delay
.def    	counter2  = r22   	;Registers used for delay
.def    	counter3  = r23   	;Registers used for delay
.def    	counter4  = r24   	;Register to set 55/AAh toggle count
.def    	pattern   = r25   	;Register that holds the LED pattern

;***** Program Start - Execution Starts Here *****
;
;***** Interrupt Vectors
    .org RESETaddr
   	  rjmp RESET        		;Reset Interrupt
    .org EXT_INT0addr				
	  rjmp EXT_INT0			;External Interrupt

; ----------------------------------------------------------------------
; Interrupt Service Routine
; ----------------------------------------------------------------------
EXT_INT0:					;External Interrupt Service Routine
	reti					;Return from interrupt after waking up 
						;from Power-Down mode 

; ----------------------------------------------------------------------
; Initialization Routine
; ----------------------------------------------------------------------
RESET:                        	;Sets up Stack Pointer and PORTD
   	cli                        	;Disable Global Interrupts                           
   	ldi 	temp,0x0f              
   	out 	SPH,temp          	;Initialize Stack Pointer 
   	ldi 	temp,0xff               
   	out 	SPL,temp          	;Initialize Stack Pointer 
   	out 	DDRD,temp         	;Configure PortD pins as outputs
   	ldi 	pattern, 0x55     	;Pattern displayed is 55 
 
   	ldi	temp, 0x01
   	out 	EIMF, temp        	;Enable External Interrupt0
   
   	ldi 	temp, (1<<SE)+(1<<SM1)+(0<<SM0) 
   	out 	MCUR, temp        	;Enable Sleep in Power Save Mode   

   	sei                        	;Enable global interrupts

; ----------------------------------------------------------------------
; Main Routine
; ----------------------------------------------------------------------
MAIN:
   	clr 	counter1          	;Initialize Counters
   	clr 	counter2
   	clr 	counter3
   	clr 	counter4
   	out 	PORTD, pattern		;Display 55h on LEDs
        
delay:                        	;Embedded delay loop to slow the display 
   	inc 	counter3	      	;of AAh & 55h pattern, making it 		
   	cpi 	counter3, 0xff		;observable on STK94 starter board
   	brne 	delay
   	clr 	counter3
   	inc 	counter2			
   	cpi 	counter2, 0xff
   	brne 	delay
   	clr 	counter2
   	inc 	counter1
   	cpi 	counter1, 0x0f
   	brne 	delay
   	clr 	counter1
   	com 	pattern           	;One's complement the pattern(55h->AAh)
   	out 	PORTD, pattern             
   	inc 	counter4
   	cpi 	counter4, 0x0A		;55/AAh toggles 10 times;
   	brne 	delay    
   	clr 	counter4
	        
	clr 	temp      	      	;LEDs are switched off to infer idle
	out 	PORTD,temp			;sleep mode on STK94 starter board
   	sleep					;Go to sleep (wait for External Interrupt)       
   	rjmp 	MAIN				;Jump to MAIN to repeat displaying 55/AAh
						;pattern after waking-up from sleep
