;***********************************************************************
;* File Name:       	avr_powersave.asm
;* Title:           	FPSLIC Sample Design 1 (Assembly)
;* Version:         	1.1
;* Last Updated:    	May 26, 2004
;* Target Device:   	AT94K05/10/40
;* Purpose:	        	To put the FPSLIC AVR Core to Sleep in Power-
;*				Save Mode and wake it upon an External Interrupt
;* Hardware used:		ATSTK94 FPSLIC Starter Kit. The Starter Board 
;*				can be used to observe the behavior of the 
;*				FPSLIC as its LED's go from flashing 55/AAh in 
;*				the active state to OFF in the sleep state 
;***********************************************************************

;***** Include Files *****
.include "AT94KDEF.INC"

;***** Global Registers *****
;* Define Symbolic Names for Registers
;* Remember immediate operations (i.e. ldi r16, 0x05) available on r16 ; ;* through r31

.def    	temp      = r16   	;Temporary Storage Register
.def    	counter1  = r21   	;Register used for delay
.def    	counter2  = r22   	;Register used for delay
.def    	counter3  = r23   	;Register used for delay
.def    	counter4  = r24   	;Register to set 55/AAh toggle count
.def    	pattern   = r25   	;Register that holds the LED pattern 
 
;***** Program Start - Execution Starts Here *****
;***** Interrupt Vectors *****
   .org RESETaddr
	   	rjmp RESET        		;Reset Interrupt

; ----------------------------------------------------------------------
; Initialization Routine
; ----------------------------------------------------------------------
RESET:                        	;Sets up Stack Pointer and PORTD
  	ldi 	temp,0x0f               
  	out 	SPH,temp          	;Initialize Stack Pointer 
  	ldi 	temp,0xff             
  	out 	SPL,temp          	;Initialize Stack Pointer 
  	out 	DDRD,temp         	;Configure PortD pins as outputs
  	ldi 	pattern, 0x55     	;Displays 55 on LEDs
 
  	ldi 	temp, (0<<WDE)+(1<<WDP2)+(1<<WDP1)+(1<<WDP0) 
  	out 	WDTCR, temp       	;Set Watchdog Timer Time-out to 1.9s                      

  	ldi 	temp, (1<<SE)+(1<<SM1)+(1<<SM0); 
  	out 	MCUR, Temp        	;Enable Sleep in Power Save Mode   

; ----------------------------------------------------------------------
; Main Routine
; ----------------------------------------------------------------------
MAIN:
  	clr 	counter1          	;Initialize counters
  	clr 	counter2
  	clr 	counter3
  	clr 	counter4
  	out 	PORTD, pattern 	  	;Display 55h on LEDs
        
delay:                        	;Embedded delay loop to slow the display 
  	inc 	counter3	      	;of AAh & 55h pattern, making it 		
  	cpi 	counter3, 0xff		;observable on STK94 starter board
  	brne 	Delay
  	clr 	counter3
  	inc 	counter2			
  	cpi 	counter2, 0xff
  	brne 	Delay
  	clr 	counter2
  	inc 	counter1
  	cpi 	counter1, 0x0f
  	brne 	Delay
  	clr 	counter1
  	com 	pattern           	;One's complement the pattern(55h->AAh)
  	out 	PORTD, pattern 
  	inc 	counter4
  	cpi 	counter4, 0x0A 		;55/AAh toggles 10 times             
  	brne 	Delay    
  	clr 	counter4
		                                   
  	clr 	temp              	;LEDs are switched off to infer idle	
  	out 	PORTD,temp			;sleep mode on STK94 starter board
  	WDR
  	in 	Temp, WDTCR			;In WDTCR value into Temp
  	ori	Temp, 0x08			;Set WDE masking WDP2, WDP1 & WDP0
  	out 	WDTCR, Temp			;Enable Watchdog Timer 
  	sleep 				;Go to sleep (wait for External Interrupt)				
